var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import { useRef, useLayoutEffect, useState, useCallback } from 'react';
var transform = function (_a) {
    var _b;
    var current = _a.current, previous = _a.previous, value = _a.value;
    if (current[0] !== current[1])
        return null;
    if (typeof current[0] !== 'number')
        return null;
    if (typeof current[1] !== 'number')
        return null;
    var start = current[0], end = current[1];
    if (start > 0 && previous[0] === start && previous[1] === start + 1) {
        return [start - 1, end];
    }
    if ((_b = value[start]) === null || _b === void 0 ? void 0 : _b.length) {
        return [start, end + 1];
    }
    return null;
};
var useCodeInputHandler = function (_a) {
    var inputRef = _a.inputRef, previousRef = _a.previousRef, setSelection = _a.setSelection;
    return useCallback(function (_a) {
        var type = _a.type;
        var input = inputRef.current;
        var previous = previousRef.current;
        if (!previous || !input)
            return;
        var direction = input.selectionDirection, value = input.value;
        var current = [input.selectionStart, input.selectionEnd];
        var save = function (selection) {
            if (selection[0] === previous[0] && selection[1] === previous[1]) {
                if (selection[0] === 0 && selection[1] === 0)
                    return;
                var DOM = [input.selectionStart, input.selectionEnd];
                if (selection[0] === DOM[0] && selection[1] === DOM[1])
                    return;
            }
            previousRef.current = selection;
            setSelection(function (state) {
                if (state[0] !== selection[0])
                    return selection;
                if (state[1] !== selection[1])
                    return selection;
                return state;
            });
            // @ts-ignore TODO: refactor SelectionState to be [number, number]
            input.setSelectionRange.apply(input, __spreadArray(__spreadArray([], selection, false), [direction], false));
        };
        if (type === 'selectionchange' && document.activeElement !== input) {
            return save([value.length, value.length]);
        }
        save(transform({ previous: previous, current: current, direction: direction, value: value }) || current);
    }, [inputRef, previousRef, setSelection]);
};
var selectionChange = function (handler) {
    var type = 'selectionchange';
    if (navigator.userAgent.includes(' Firefox/')) {
        var interval_1 = setInterval(handler, 16, { type: type });
        return function () { return clearInterval(interval_1); };
    }
    document.addEventListener(type, handler);
    return function () { return document.removeEventListener(type, handler); };
};
var useCodeInputEffect = function (_a) {
    var inputRef = _a.inputRef, previousRef = _a.previousRef, handler = _a.handler;
    useLayoutEffect(function () {
        var input = inputRef.current;
        if (previousRef.current === undefined && input) {
            previousRef.current = [input.selectionStart, input.selectionEnd];
        }
        var cleanupSelectionChange = selectionChange(handler);
        input === null || input === void 0 ? void 0 : input.addEventListener('input', handler);
        return function () {
            cleanupSelectionChange();
            input === null || input === void 0 ? void 0 : input.removeEventListener('input', handler);
        };
    }, [inputRef, handler, previousRef]);
};
export var useCodeInput = function (inputRef) {
    var _a = useState([0, 0]), selection = _a[0], setSelection = _a[1];
    var previousRef = useRef();
    var handler = useCodeInputHandler({
        inputRef: inputRef,
        previousRef: previousRef,
        setSelection: setSelection
    });
    useCodeInputEffect({ inputRef: inputRef, previousRef: previousRef, handler: handler });
    return selection;
};
